using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace Damage
{
    public class AndroidDamageBehaviour : MonoBehaviour, Messages.IShot, Messages.IRepair
    {
        public LimbBehaviour limb;
        public PersonBehaviour person;

        public AndroidOrgan[] organs;
        public bool hasOrgan;
        public bool vitalOrganDestroyed;

        public float armorStrength; // armoring around the limbs, greatly reduces strength of bullets and is reduced on hit
        public float initialArmor;

        public float initialStrength;
        public float motorChance; // chance for the motor to be hit, lowering strength

        public float oil; // how much oil has leaked. more oil means higher chance of combustion on getting shot and lower functioning of organs. out of 5.6 (2 liters)

        bool detailMode;

        void Start()
        {
            limb = GetComponent<LimbBehaviour>();
            person = limb.Person;
            limb.CirculationBehaviour.ImmuneToDamage = true;
            
            switch (gameObject.name)
            {
                case "Head":
                    organs = new AndroidOrgan[]
                    {
                        AndroidOrgan.CreateOrgan("Computer", health: 350, integrity: 200, chance: 1f, canLeak: false, vital: true)
                    };
                    armorStrength = 175f;
                    break;
                case "UpperBody":
                    organs = new AndroidOrgan[]
                    {
                        AndroidOrgan.CreateOrgan("Engine", health: 500, integrity: 350, chance: 0.8f, canLeak: true, vital: true),
                        AndroidOrgan.CreateOrgan("Pump", health: 300, integrity: 200, chance: 0.3f, canLeak: true, vital: false) // if the pump is destroyed, oil levels should begin to decrease
                    };
                    armorStrength = 125f;
                    break;
                case "MiddleBody":
                case "LowerBody":
                    organs = new AndroidOrgan[]
                    {
                        AndroidOrgan.CreateOrgan("Oil tanks", health: 250, integrity: 50, chance: 0.9f, canLeak: true, vital: false) // body functioning should be reduced when these are damaged
                    };
                    armorStrength = 200f;
                    break;
                default:
                    armorStrength = 50f;
                    motorChance = 0.35f;
                    break;
            }
            if (organs != null)
                hasOrgan = true;
            initialArmor = armorStrength;
            initialStrength = limb.BaseStrength;
        }
        void Awake()
        {
            Global.main.LimbStatusToggled += new EventHandler<bool>(Main_LimbStatusToggled);
        }
        void OnDestroy()
        {
            Global.main.LimbStatusToggled -= new EventHandler<bool>(Main_LimbStatusToggled);
        }
        private void Main_LimbStatusToggled(object sender, bool e)
        {
            detailMode = e;
            if (!e)
            {
                SpriteRenderer sr = GetComponent<SpriteRenderer>();
                Color color = sr.color;
                color.r = 1;
                color.g = 1;
                color.b = 1;
                sr.color = color;
            }
        }
        void Update()
        {
            limb.CirculationBehaviour.InternalBleedingIntensity = 0f;
        }
        void FixedUpdate()
        {
            if (hasOrgan)
            {
                foreach (AndroidOrgan organ in organs)
                {
                    if (organ.vital)
                    {
                        limb.Health = limb.InitialHealth * (organ.health / organ.initialHealth);
                    }
                    if (organ.canLeak && organ.leakRate > 0)
                    {
                        float rate = organ.leakRate * limb.CirculationBehaviour.BloodFlow * (1 - (organ.integrity / organ.initialIntegrity));
                        oil += rate * Time.deltaTime;
                        oil = Mathf.Min(oil, 5.6f);
                        limb.CirculationBehaviour.Drain(rate / 4f * Time.deltaTime);

                        organ.leakRate -= 0.00933333333f * Time.deltaTime;
                    }
                    if (organ.name == "Computer")
                    {
                        foreach (LimbBehaviour limb in person.Limbs)
                        {
                            limb.Numbness = Mathf.Max(limb.Numbness, (1 - (organ.health / organ.initialHealth)) * 0.5f);
                        }
                    }
                    if (!organ.destroyed)
                    {
                        organ.SetHealth(Mathf.Min(organ.health, organ.initialHealth * (1 - (oil / 7f))));
                        if (organ.vital)
                            limb.CirculationBehaviour.IsPump = limb.CirculationBehaviour.WasInitiallyPumping;
                        if (limb.PhysicalBehaviour.Charge > 0 && (organ.name == "Computer" || organ.name == "Pump")) // only electrical components
                            organ.Damage(limb.PhysicalBehaviour.Charge * 0.01f);
                        if (limb.InternalTemperature > limb.DiscomfortingHeatTemperature)
                            organ.Damage(1 / 400f * (limb.InternalTemperature - limb.DiscomfortingHeatTemperature));
                        if (organ.canLeak && limb.InternalTemperature < 16f)
                            organ.Damage(1 / 200f); // oil freezes
                    }
                    else
                    {
                        if (organ.vital && !vitalOrganDestroyed)
                        {
                            KillBody();
                        }
                        if (organ.name == "Pump")
                        {
                            limb.CirculationBehaviour.Drain(0.233333333f * Time.deltaTime);
                        }
                    }
                }
            }
            if (detailMode && oil > 0)
            {
                SpriteRenderer sr = GetComponent<SpriteRenderer>();
                Color color = sr.color;
                color.r = (oil / -11.2f) + 1;
                color.g = (oil / -11.2f) + 1;
                color.b = (oil / -11.2f) + 1;
                sr.color = color;
            }
        }
        public void Shot(Shot shot)
        {
            ++limb.CirculationBehaviour.GunshotWoundCount;
            float damage = shot.damage / 6f;
            if (damage > armorStrength)
            {
                damage -= armorStrength;

                if (UnityEngine.Random.value < motorChance)
                {
                    limb.BaseStrength -= Mathf.Min(shot.damage / 12f, 0.5f);
                    limb.BaseStrength = Mathf.Max(limb.BaseStrength, 2f);
                }

                if (hasOrgan)
                {
                    foreach (AndroidOrgan organ in organs)
                    {
                        float chance = UnityEngine.Random.value;
                        if (chance <= organ.chance)
                        {
                            organ.DamageIntegrity(damage);
                        }
                    }    
                }

                if (damage > 1 && UnityEngine.Random.value <= oil / 5.6f * 0.05f)
                {
                    limb.Crush();
                    ExplosionCreator.CreateFragmentationExplosion(new ExplosionCreator.ExplosionParameters(8U, transform.position, 3, 8f, true, big: false, dismember: 0f, shrapnelCount: 8));
                }
            }
            float a = armorStrength;
            armorStrength -= shot.damage / 6f;
            armorStrength = Mathf.Max(armorStrength, 0);
            //if (armorStrength > 0)
                //Debug.Log("Armor strength reduced to " + armorStrength + " from " + a);
        }
        public void Repair()
        {
            limb.BaseStrength = initialStrength;
            armorStrength = initialArmor;
            oil = 0;
            if (hasOrgan)
            {
                foreach (AndroidOrgan organ in organs)
                {
                    organ.SetHealth(organ.initialHealth);
                    organ.leakRate = 0;
                    organ.integrity = organ.initialIntegrity;
                    organ.destroyed = false;
                }
            }
        }
        void KillBody()
        {
            foreach (LimbBehaviour limb in person.Limbs)
            {
                if (limb.TryGetComponent(out AndroidDamageBehaviour damage))
                {
                    if (damage.hasOrgan)
                    {
                        foreach (AndroidOrgan organ in damage.organs)
                        {
                            organ.health = 0;
                        }
                    }
                    vitalOrganDestroyed = true;
                    limb.CirculationBehaviour.IsPump = false;
                }
            }
        }
    }
    public class AndroidOrgan : ScriptableObject
    {
        public static AndroidOrgan CreateOrgan(string name, float health, float integrity, float chance, bool canLeak = true, bool vital = false, float multiplier = 1)
        {
            AndroidOrgan organ = CreateInstance<AndroidOrgan>();
            organ.name = name;
            organ.vital = vital;
            organ.health = health;
            organ.actualHealth = health;
            organ.initialHealth = health;
            organ.integrity = integrity;
            organ.initialIntegrity = integrity;
            organ.chance = chance;
            organ.canLeak = canLeak;
            organ.leakMultiplier = multiplier;
            return organ;
        }
        public string name;

        public bool vital;
        public bool destroyed = false;

        public float health;
        public float initialHealth;

        public float actualHealth; // so health can be shifted around

        public float integrity;
        public float initialIntegrity;

        public float chance;

        public bool canLeak;
        public float leakMultiplier;
        public float leakRate = 0;

        public void Damage(float damage)
        {
            health -= damage;
            actualHealth -= damage;
            if (health <= 0)
            {
                destroyed = true;
                //Debug.Log(name + " destroyed");
            }
            health = Mathf.Max(0, health);
            actualHealth = Mathf.Max(0, actualHealth);
            //Debug.Log(name + " damaged by " + damage + ". Health is now at " + health);
        }
        public void DamageIntegrity(float damage)
        {
            Damage(damage);
            integrity -= damage;
            if (canLeak)
            {
                leakRate += Mathf.Min(damage / 260f) * leakMultiplier;
                leakRate = Mathf.Min(leakRate, 0.28f * leakMultiplier);
                //Debug.Log("Leak rate is now at " + leakRate);
            }
            integrity = Mathf.Max(0, integrity);
        }
        public void SetHealth(float num, float shift = 0)
        {
            health = num + shift;
            actualHealth = num;
        }
    }
}